"""Add a buffer around the queried tile from its neighbors
The script assumes that the neighbor tiles are located in the same folder as
the queried tile

"""

import logging
import os

import hydra
from omegaconf import DictConfig
from pdaltools.las_add_buffer import create_las_with_buffer

from las_digital_models.commons import commons

log = commons.get_logger(__name__)


@hydra.main(config_path="../configs/", config_name="config.yaml", version_base="1.2")
def run_add_buffer_one_tile(config: DictConfig):
    """
    The script assumes that the neighbor tiles are located in the same folder as
    the queried tile
    """

    if config.io.forced_intermediate_ext is None:
        input_file = os.path.join(config.io.input_dir, config.io.input_filename)
        output_file = os.path.join(config.io.output_dir, config.io.input_filename)
    else:
        _, input_basename = os.path.split(config.io.input_filename)
        tilename, _ = os.path.splitext(input_basename)
        input_file = os.path.join(config.io.input_dir, f"{tilename}.{config.io.forced_intermediate_ext}")
        output_file = os.path.join(config.io.output_dir, f"{tilename}.{config.io.forced_intermediate_ext}")

    os.makedirs(config.io.output_dir, exist_ok=True)

    create_las_with_buffer(
        input_dir=config.io.input_dir,
        tile_filename=input_file,
        output_filename=output_file,
        buffer_width=config.buffer.size,
        spatial_ref=config.io.spatial_reference,
        tile_width=config.tile_geometry.tile_width,
        tile_coord_scale=config.tile_geometry.tile_coord_scale,
    )


def main():
    logging.basicConfig(level=logging.INFO)
    run_add_buffer_one_tile()


if __name__ == "__main__":
    main()



""" Main script for interpolation on a single tile
Output files will be written to the target folder, tagged with the name of the interpolation
method that was used.
"""

import logging
import os
import tempfile

import hydra
from omegaconf import DictConfig

from las_digital_models.commons import commons
from las_digital_models.tasks.las_interpolation import interpolate_from_config
from las_digital_models.tasks.postprocessing import mask_with_no_data_shapefile

log = commons.get_logger(__name__)


@hydra.main(config_path="../configs/", config_name="config.yaml", version_base="1.2")
def run_ip_on_tile(config: DictConfig):
    """Run interpolation on single tile using hydra config
    config parameters are explained in the default.yaml files
    """
    if config.io.input_dir is None:
        input_dir = config.io.output_dir
    else:
        input_dir = config.io.input_dir

    os.makedirs(config.io.output_dir, exist_ok=True)
    tilename, _ = os.path.splitext(config.io.input_filename)

    # input file (already filtered and potentially with a buffer)
    if config.io.forced_intermediate_ext is None:
        input_file = os.path.join(input_dir, config.io.input_filename)
    else:
        input_file = os.path.join(input_dir, f"{tilename}.{config.io.forced_intermediate_ext}")

    # for export
    _size = commons.give_name_resolution_raster(config.tile_geometry.pixel_size)
    geotiff_stem = f"{tilename}{_size}"
    geotiff_filename = f"{geotiff_stem}.tif"
    geotiff_path = os.path.join(config.io.output_dir, geotiff_filename)

    if config.io.no_data_mask_shapefile:
        with tempfile.NamedTemporaryFile(suffix=".tif", prefix=f"{geotiff_stem}_raw") as tmp_geotiff:
            # process interpolation
            interpolate_from_config(input_file, tmp_geotiff.name, config)
            mask_with_no_data_shapefile(
                config.io.no_data_mask_shapefile, tmp_geotiff.name, geotiff_path, config.tile_geometry.no_data_value
            )

    else:
        interpolate_from_config(input_file, geotiff_path, config)


def main():
    logging.basicConfig(level=logging.INFO)
    run_ip_on_tile()


if __name__ == "__main__":
    main()



import logging
import os
import shutil
import test.utils.point_cloud_utils as pcu

from hydra import compose, initialize

from las_digital_models import add_buffer_one_tile

TEST_PATH = os.path.dirname(__file__)
TMP_PATH = os.path.join(TEST_PATH, "tmp/buffer")
DATA_PATH = os.path.join(TEST_PATH, "data")


def setup_module(module):
    try:
        shutil.rmtree(TMP_PATH)

    except FileNotFoundError:
        pass
    os.makedirs(TMP_PATH, exist_ok=True)


def test_add_buffer_one_tile():
    # DATA_PATH contains a .laz file, but check that the io.forced_intermediate_ext parameter forces to look for a
    # .laz file
    input_filename = "test_data_77055_627760_LA93_IGN69.las"
    output_filename = "test_data_77055_627760_LA93_IGN69.laz"
    output_file = os.path.join(TMP_PATH, output_filename)

    with initialize(version_base="1.2", config_path="../configs"):
        # config is relative to a module
        cfg = compose(
            config_name="config",
            overrides=[
                "io=test",
                "tile_geometry=test",
                f"io.input_dir={DATA_PATH}",
                f"io.input_filename={input_filename}",
                "io.forced_intermediate_ext=laz",
                f"io.output_dir={TMP_PATH}",
                "buffer=test",
            ],
        )

    add_buffer_one_tile.run_add_buffer_one_tile(cfg)
    assert os.path.isfile(output_file)
    logging.info(pcu.get_nb_points(output_file))
    assert pcu.get_nb_points(output_file) == 103359
    assert pcu.get_classification_values(output_file) == {1, 2, 3, 4, 5, 6, 64}


if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)
    test_add_buffer_one_tile()



from typing import List

import pdal
from osgeo import gdal
from pdaltools.las_info import parse_filename

from las_digital_models.commons import commons

gdal.UseExceptions()


def interpolate_from_config(input_file: str, output_raster: str, config: dict):
    """API using a config dictionary for the `interpolate` method defined in this file
    Generate a Z (height) raster file from a LAS point cloud file by interpolating the Z value for each pixel center.


    Args:
        input_file (str): path to the las/laz file to interpolate
        output_file (str): path to the output raster
        config (dict): ProduitDeriveLidar config dictionary containing
        {
            "tile_geometry": {
                "tile_coord_scale": #int, scale of the tiles coordinates in the las filename
                "tile_width": #int, width of the tile in meters (used to infer the lower-left corner)
                "pixel_size": #float, pixel size of the output raster in meters (pixels are supposed to be squares)
                "no_data_value": #int, no data value for the output raster
            },
            "io": {
                "spatial_reference": #str, spatial reference to use when reading las file
            },
            "filter": {
                "dimension": #str, dimension alogn which to filter
                "keep_values": #list of ints, values of the gilter dimension for the points to use in the interpolation
            }
        }

    """
    interpolate(
        input_file,
        output_raster,
        config["tile_geometry"]["pixel_size"],
        config["tile_geometry"]["tile_width"],
        config["tile_geometry"]["tile_coord_scale"],
        config["io"]["spatial_reference"],
        config["tile_geometry"]["no_data_value"],
        config["filter"]["dimension"],
        config["filter"]["keep_values"],
    )


@commons.eval_time_with_pid
def interpolate(
    input_file: str,
    output_file: str,
    pixel_size: float,
    tile_width: int,
    tile_coord_scale: int,
    spatial_ref: str,
    no_data_value: int,
    filter_dimension: str,
    filter_values: List[int],
):
    """Generate a Z (height) raster file from a LAS point cloud file by interpolating the Z value at the center of
    each pixel.

    It uses TIN interpolation.

    Steps are:
    - filter the points to use in the interplation (using one dimension name and a list of values)
    (eg. Classification=2(ground) for a digital terrain model)
    - triangulate the point cloud using Delaunay
    - interpolate the height values at the center of the pixels using Faceraster
    - write the result in a raster file.

    Args:
        input_file (str): path to the las/laz file to interpolate
        output_file (str): path to the output raster
        pixel_size (float): pixel size of the output raster in meters (pixels are supposed to be squares)
        tile_width (int): width of the tile in meters (used to infer the lower-left corner)
        tile_coord_scale (int): scale of the tiles coordinates in the las filename
        spatial_ref (str): spatial reference to use when reading las file
        no_data_value (int): no data value for the output raster
        filter_dimension (str): Name of the dimension along which to filter input points
        (keep empty to disable input filter)
        filter_values (List[int]): Values to keep for input points along filter_dimension
    """

    _, coordX, coordY, _ = parse_filename(input_file)

    # Compute origin/number of pixels
    origin = [float(coordX) * tile_coord_scale, float(coordY) * tile_coord_scale]
    nb_pixels = [int(tile_width / pixel_size), int(tile_width / pixel_size)]

    # Read with pdal
    pipeline = pdal.Reader.las(filename=input_file, override_srs=spatial_ref, nosrs=True)
    if filter_dimension and filter_values:
        pipeline |= pdal.Filter.range(limits=",".join(f"{filter_dimension}[{v}:{v}]" for v in filter_values))

    # Add interpolation method to the pdal pipeline
    pipeline |= pdal.Filter.delaunay()

    pipeline |= pdal.Filter.faceraster(
        resolution=str(pixel_size),
        origin_x=str(origin[0] - pixel_size / 2),  # lower left corner
        origin_y=str(origin[1] + pixel_size / 2 - tile_width),  # lower left corner
        width=str(nb_pixels[0]),
        height=str(nb_pixels[1]),
    )
    pipeline |= pdal.Writer.raster(gdaldriver="GTiff", nodata=no_data_value, data_type="float32", filename=output_file)

    pipeline.execute()
